package controllers;

import java.util.Date;
import java.util.Scanner;

import models.Butter;
import models.Milk;
import models.Product;
import static utils.ScannerInput.*;

/**
 * This class runs the application and handles the Store with its Products.
 *  
 * @author Siobhan Drohan
 */
public class MenuController{

	private Scanner input;
	private Store store;

	public static void main(String[] args){
		new MenuController();
	}

	public MenuController(){
		input = new Scanner(System.in);

		//read in the details....
		System.out.print("Please enter the store location: ");
		String location = input.nextLine();

		store = new Store(location);
		runMenu();
	}

	/*
	 * mainMenu() - This method displays the menu for the application, 
	 * reads the menu option that the user entered and returns it.
	 * 
	 * @return     the users menu choice
	 */
	private int mainMenu()
	{ 
		System.out.println("\fShop Menu");
		System.out.println("---------");     
		System.out.println("  1) Add a Product");    
		System.out.println("  2) List the Products"); 
		System.out.println("  3) Update a Product");    
		System.out.println("  4) Remove Product (by index)");        
		System.out.println("---------");             
		System.out.println("  5) List the cheapest product");        
		System.out.println("---------");     
		System.out.println("  6) View store details"); 
		System.out.println("---------");     
		System.out.println("  7) Save products (XML)"); 
		System.out.println("  8) Load products (XML)"); 
		System.out.println("  0) Exit");
		int option = validNextInt("==>> ");
		return option;
	}

	/*
	 * This is the method that controls the loop.
	 */
	private void runMenu(){
		int option = mainMenu();
		while (option != 0){  
			switch (option){
			case 1:    addProduct();
			break;
			case 2:    System.out.println(store.listProducts());
			break;
			case 3:    updateProduct();
			break;              
			case 4:    store.remove(getIndex());
			break;              
			case 5:    System.out.println(store.cheapestProduct());
			break;
			case 6:    System.out.println(store.toString());
			break;
			case 7:    try {
				store.save();
			}
			catch (Exception e) {
				System.err.println("Error writing to file: " + e);
			}
			break;
			case 8:    try {
				store.load();
			}
			catch (Exception e) {
				System.err.println("Error reading from file: " + e);
			}
			break;
			default:   System.out.println("Invalid option entered: " + option);
			break;
			}
			//pause the program so that the user can read what we just printed to the terminal window
			System.out.println("\nPress any key to continue...");
			input.nextLine();
			//display the main menu again
			option = mainMenu();
		}
		//the user chose option 0, so exit the program
		System.out.println("Exiting... bye");
		System.exit(0);
	}

	//gather the product data from the user and create a new product.
	private void addProduct(){  
		store.add(readProductDetails());
	}

	private void updateProduct() {
		//retrieve the index for the product for update
		int index = getIndex();
		if (store.isValidIndex(index)){
			//gather details to update the selected product with
			Product productDetails = readProductDetails();

			//retrieve the product and update it using the details entered in by the user
			Product productToUpdate = store.get(index);
			productToUpdate.setInCurrentProductLine(productDetails.isInCurrentProductLine());
			productToUpdate.setProductCode(productDetails.getProductCode());
			productToUpdate.setProductName(productDetails.getProductName());
			productToUpdate.setUnitCost(productDetails.getUnitCost());
		}
	}

	//===========================
	// HELPER / UTILITY METHODS
	//===========================

	private Product readProductDetails() {
		//read the product details from the user and return them as a product object
		System.out.println("Enter the Product details...");
		System.out.print("\tName:  ");
		String productName = input.nextLine();

		int productCode = validNextInt("\tCode (between 1000 and 9999):  ");
		double unitCost = validNextDouble("\tUnit Cost:  ");
		
		System.out.print("\tIs this product in your current line (y/n): ");
		char currentProduct = input.next().charAt(0);
		boolean inCurrentProductLine = false;
		if ((currentProduct == 'y') || (currentProduct == 'Y'))
			inCurrentProductLine = true;

		Date bestBeforeDate = readValidDate("\tBest before date (dd/MM/yyyy): ");
		
		System.out.print("\tIs this product butter or milk (b/m): ");
		char butterOrMilk = input.next().charAt(0);
		boolean isButterOrMilk = false;
		if ((butterOrMilk == 'b') || (butterOrMilk == 'B')) {
			int packetGrams = validNextInt("\tPacket Grams:  ");
			System.out.print("\tIs this butter salted(y/n): ");
			char salted = input.next().charAt(0);
			boolean isSalted = false;
			if ((salted == 'y') || (salted == 'Y'))
				isSalted = true;
			return (new Butter(productName, productCode, unitCost, inCurrentProductLine, 
					bestBeforeDate, packetGrams, isSalted));
		}
		else if ((butterOrMilk == 'm') || (butterOrMilk == 'M')) {
			int cartonSize = validNextInt("\tCarton Size (mls):  ");
			System.out.print("\tFat content (full, low, skimmed):  ");
			String fatContent = input.nextLine();	
			fatContent = input.nextLine();	
		    return (new Milk(productName, productCode, unitCost, inCurrentProductLine,
		    		bestBeforeDate, cartonSize, fatContent));
		}
		else {
			return null;
		}
	}

	private int getIndex(){
		System.out.println(store.listProducts());
		if (store.size() > 0){
			int index = validNextInt("Please enter the index to select the product: ");
			if (store.isValidIndex(index)){
				return index;
			}
			else{
				System.out.println("Invalid index entered: " + index ); 
				return -1;  //error code - invalid index entered
			}
		}
		else {
			return -2;  //error code - empty array list
		}
	}
	
}
